"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Implementation if {@link Result} that resolves when an {@link Emitter} emits a value
 *
 * If the {@link Emitter} emits an `error` event, the Result is rejected.
 */
var ResultImpl = /** @class */ (function () {
    /**
     * Create a `ResultImpl` with an emitter.
     *
     * @param emitter the emitter that this Result will listen to
     */
    function ResultImpl(emitter) {
        this.promise = new Promise(function (fulfilled, rejected) {
            emitter.listen(function (e, args) {
                if (e === 'error') {
                    rejected(args[0]);
                }
                else {
                    fulfilled(args[0]);
                }
            });
        });
    }
    /**
     * Forward the call to the internal `promise`
     *
     * @inheritdoc
     */
    ResultImpl.prototype.then = function (fulfilled, rejected) {
        return this.promise.then(fulfilled, rejected);
    };
    /**
     * Forward the call to the internal `promise`
     *
     * @inheritdoc
     */
    ResultImpl.prototype.catch = function (rejected) {
        return this.promise.catch(rejected);
    };
    /**
     * Forward the call to the internal `promise`
     *
     * @inheritdoc
     */
    ResultImpl.prototype.finally = function (onFinally) {
        return this.promise.finally(onFinally);
    };
    return ResultImpl;
}());
exports.ResultImpl = ResultImpl;
