"use strict";
/**
 * @module ValueStream
 */
Object.defineProperty(exports, "__esModule", { value: true });
var datatypes_1 = require("data/datatypes");
var time_series_event_1 = require("services/timeseries/time-series-event");
/**
 * Adapter for transforming the type of the emitted value to the datatype of
 * the ValueStream.
 */
var ValueStreamTimeseriesAdapter = /** @class */ (function () {
    /**
     * Create a new ValueStreamAdapter instance
     *
     * @param stream        the stream that is wrapped
     * @param datatype      the data type of the value stream
     * @param topictype     the topic type
     */
    function ValueStreamTimeseriesAdapter(stream, datatype, topicSpec) {
        this.stream = stream;
        this.datatype = datatype;
        this.sourceDatatype = datatypes_1.DataTypes.timeseries(datatypes_1.DataTypes.getByValue(topicSpec.properties.TIME_SERIES_EVENT_VALUE_TYPE));
    }
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.selects = function (specification) {
        return this.stream.selects(specification);
    };
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.onOpen = function () {
        return this.stream.onOpen();
    };
    ValueStreamTimeseriesAdapter.prototype.readAsDataType = function (buffer) {
        var sourceEvent = this.sourceDatatype.readValue(buffer);
        var sourceValuetype = datatypes_1.DataTypes.getByName(this.sourceDatatype.valueTypeName);
        var destValuetype = datatypes_1.DataTypes.getByName(this.datatype.valueTypeName);
        var destValue = sourceValuetype.readAs(destValuetype.valueClass, sourceValuetype.writeValue(sourceEvent.value));
        return new time_series_event_1.EventImpl(sourceEvent.metadata, sourceEvent.originalEvent, destValue);
    };
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.onDelta = function (path, specification, oldContent, content, delta, oldValue, newValue) {
        return this.stream.onDelta(path, specification, oldContent, content, delta, (oldContent !== null) ? this.readAsDataType(oldContent) : null, this.readAsDataType(content));
    };
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.onValue = function (path, specification, oldContent, content, oldValue, newValue) {
        return this.stream.onValue(path, specification, oldContent, content, (oldContent !== null) ? this.readAsDataType(oldContent) : null, this.readAsDataType(content));
    };
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.onSubscription = function (path, specification) {
        return this.stream.onSubscription(path, specification);
    };
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.onUnsubscription = function (path, specification, reason) {
        return this.stream.onUnsubscription(path, specification, reason);
    };
    /**
     * @inheritdoc
     */
    ValueStreamTimeseriesAdapter.prototype.onSubscriptionError = function (error) {
        return this.stream.onSubscriptionError(error);
    };
    return ValueStreamTimeseriesAdapter;
}());
exports.ValueStreamTimeseriesAdapter = ValueStreamTimeseriesAdapter;
