"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Callbacks for services for each session tab
 */
var WorkerServiceCallbacks = /** @class */ (function () {
    function WorkerServiceCallbacks() {
        this.callbacks = {};
    }
    /**
     * Add a service callback from a session tab
     *
     * @param  serviceId  the service ID
     * @param  respond    the worker response that sends the command back to the tab
     */
    WorkerServiceCallbacks.prototype.addService = function (serviceId, respond) {
        // tslint:disable-next-line:strict-type-predicates
        if (this.callbacks[serviceId] === undefined) {
            this.callbacks[serviceId] = [respond];
        }
        else {
            this.callbacks[serviceId].push(respond);
        }
    };
    /**
     * Call service callbacks for all registered session tabs
     *
     * @param  serviceId  the service ID
     * @param  request    the serialised request from the server
     * @param  rid        the request ID
     */
    WorkerServiceCallbacks.prototype.onRequest = function (serviceId, request, rid) {
        var e_1, _a;
        // tslint:disable-next-line:strict-type-predicates
        if (this.callbacks[serviceId] !== undefined) {
            try {
                for (var _b = __values(this.callbacks[serviceId]), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var respond = _c.value;
                    respond(['service_request', serviceId.toString(10), rid.toString(), request.toString('base64')]);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
    };
    return WorkerServiceCallbacks;
}());
exports.WorkerServiceCallbacks = WorkerServiceCallbacks;
